import { AnimatePresence, motion, useDragControls } from "framer-motion";
import { useEffect, useState } from "react";
import { sendSignInCode, signIn } from "../../telegram/telegram";
import {
  login,
  setCodeHash,
  useSelectCodeHash,
  useSelectorIsLoggedIn,
} from "../../store/user";
import { useDispatch } from "react-redux";
import { Link } from "react-router-dom";
import { getMe } from "../../telegram/user";
import Uploader from "../../commponse/uploader";

export default function Home() {
  let [hiddenDrawer, setHiddenDrawer] = useState(true);
  let dispatch = useDispatch();
  let handleHiddenDrawer = (e: MouseEvent) => {
    if (!hiddenDrawer) {
      console.log(e.target);
      setHiddenDrawer(true);
    }
  };

  useEffect(() => {
    document.addEventListener("click", handleHiddenDrawer);
    return () => document.removeEventListener("click", handleHiddenDrawer);
  });

  useEffect(() => {
    getMe()
      .then((user) => {
        dispatch(login(user));
      })
      .catch((e) => {
        console.log(e);
      });
  }, []);

  return (
    <div className={"h-screen w-screen bg-amber-400"}>
      <FileButton openLogin={() => setHiddenDrawer(false)} />
      <Drawer
        hidden={hiddenDrawer}
        setHidden={() => {
          setHiddenDrawer(true);
        }}
      />
      <Menu
        onClickLogin={() => {
          setHiddenDrawer(!hiddenDrawer);
        }}
      />
      <Logo />
    </div>
  );
}

function Logo() {
  return (
    <motion.div
      className={
        "absolute top-0 m-4 box-border h-8 w-24 cursor-pointer select-none border border-4 border-base-300 bg-white bg-base-100 text-center shadow-lg"
      }
    >
      LOGO
    </motion.div>
  );
}

function Menu(props: { onClickLogin: () => void }) {
  let loggedIn = useSelectorIsLoggedIn();
  return (
    <motion.ul
      initial={{
        y: 20,
        opacity: 0,
      }}
      animate={{
        y: 0,
        opacity: 1,
      }}
      exit={{
        y: -20,
        opacity: 0,
      }}
      transition={{ duration: 0.3 }}
      className={
        "absolute right-8 top-0 m-4 flex justify-end gap-4 rounded-l bg-base-200 px-8 py-2 shadow"
      }
    >
      <li
        className={
          "cursor-pointer select-none p-2 text-base hover:bg-amber-400 hover:bg-opacity-50"
        }
      >
        定价
      </li>
      {!loggedIn ? (
        <li
          className={
            "cursor-pointer select-none p-2 text-base hover:text-gray-500"
          }
          onClick={props.onClickLogin}
        >
          注册/登录
        </li>
      ) : (
        <Link to={"dashboard"}>
          <li
            className={
              "cursor-pointer select-none p-2 text-base hover:text-gray-500"
            }
          >
            文件列表
          </li>
        </Link>
      )}
    </motion.ul>
  );
}

function FileButton(props: { openLogin: () => void }) {
  return (
    <div className={"btn-group absolute top-1/2 left-1/4"}>
      <Uploader>
        <button className={"btn rounded-l-full"}>上传文件</button>
      </Uploader>
      <button className={"btn rounded-r-full"} onClick={props.openLogin}>
        口令下载
      </button>
    </div>
  );
}

// Drawer 侧边栏
function Drawer(props: { hidden: boolean; setHidden: () => void }) {
  let width = window.innerWidth;
  let control = useDragControls();
  return (
    <AnimatePresence>
      {props.hidden ? (
        ""
      ) : (
        <motion.div
          exit={{ opacity: 0 }}
          className={"h-screen w-screen overflow-hidden"}
        >
          <div
            className={
              "fixed left-0 top-0 h-screen w-screen bg-black opacity-20"
            }
          />
          <motion.div
            initial={{ translateX: width / 2 }}
            animate={{ translateX: 0 }}
            exit={{ translateX: width / 2 }}
            transition={{ duration: 0.3 }}
            dragControls={control}
            drag={"x"}
            dragListener={false}
          >
            <div
              className={
                "absolute right-0 flex h-screen w-1/2 origin-right justify-center bg-base-100"
              }
              onClick={(e) => {
                e.nativeEvent.stopImmediatePropagation();
              }}
            >
              <div
                className={
                  "flex h-full w-3/4 flex-col items-center justify-center gap-8"
                }
              >
                <div
                  onPointerDown={(event) => {
                    console.log(event);
                    control.start(event, { snapToCursor: false });
                  }}
                  className={"absolute top-0 h-20 w-full bg-amber-200"}
                />
                <LoginContent setHidden={props.setHidden} />
              </div>
            </div>
          </motion.div>
        </motion.div>
      )}
    </AnimatePresence>
  );
}

function LoginContent(props: { setHidden: () => void }) {
  let [phone, setPhone] = useState("");
  let [phoneCode, setPhoneCode] = useState("");
  let codeHash = useSelectCodeHash();
  const dispatch = useDispatch();

  let handleSendCode = () => {
    sendSignInCode(phone)
      .then((hash) => {
        dispatch(setCodeHash(hash));
      })
      .catch((e) => {
        alert(e.error_message);
      });
  };

  let handleLogin = () => {
    signIn(phone, phoneCode, codeHash)
      .then((e) => {
        dispatch(login(e.user));
        props.setHidden();
        alert("登录成功");
      })
      .catch((e) => {
        alert(e.error_message);
      });
  };

  return (
    <div className={"flex flex-col gap-8 rounded-2xl border-2 p-16"}>
      {codeHash === "" ? (
        <>
          <div className={"flex bg-base-100"}>
            <label className={"label mr-5 text-base-content"}>手机号</label>
            <input
              className={"input input-primary"}
              value={phone}
              onChange={(e) => setPhone(e.target.value)}
            />
          </div>
          <button className={"btn self-end"} onClick={handleSendCode}>
            发送验证码
          </button>
        </>
      ) : (
        <div className={"flex flex-col gap-8"}>
          <label className={"label mr-5 text-base-content"}>验证码</label>
          <input
            className={"input input-primary"}
            value={phoneCode}
            onChange={(e) => setPhoneCode(e.target.value)}
          />
          <button className={"btn self-end"} onClick={handleLogin}>
            登录
          </button>
        </div>
      )}
    </div>
  );
}
